S3method(plot,autotune_lasso)
S3method(coef,autotune_lasso)
S3method(coef,autotune_lasso_path)
S3method(predict,autotune_lasso)
export(autotune_lasso)
importFrom(Rcpp, evalCpp)
useDynLib(autotune, .registration = TRUE)
importFrom("graphics", "abline", "grid", "legend", "lines", "par", "points", "text")
importFrom("stats", "coef", "lm")
importFrom("utils", "tail")
