# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

autotune_lasso_cpp <- function(xin, yin, alpha = 0.01, standardize = TRUE, standardize_response = TRUE, intercept = TRUE, active = FALSE, trace_it = FALSE, tolerance = 1e-4, beta_tolerance = 1e-3, iter_max = 30L, beta_iter_max = 40L, active_iter_max = 5L, PR_norm_l2 = FALSE, tau = 0.5) {
    .Call(`_autotune_autotune_lasso_cpp`, xin, yin, alpha, standardize, standardize_response, intercept, active, trace_it, tolerance, beta_tolerance, iter_max, beta_iter_max, active_iter_max, PR_norm_l2, tau)
}

