\name{sp500}
\alias{sp500}
\docType{data}
\title{S&P 500 Stock Data}
\description{
The \code{sp500} data file contains a year's worth of close-of-day data
for most of the Standard and Poor's 500 stocks. The data are in reverse
chronological order, with the first row corresponding to December 31,
2008.
}
\usage{
data("sp500")
}
\format{
A list containing the following two matrices:
\describe{
  \item{\code{sp500.percent}}{
    A 252 by 494 matrix of daily percentage changes. The first
    column is the DJIA index, the second is the S&P 500 index, and the
    remaining columns are labeled individual stocks.
  }
  \item{\code{sp500.2008}}{
    A 253 by 494 matrix of raw close-of-day data. The first
    column is the DJIA index, the second is the S&P 500 index, and the
    remaining columns are labeled individual stocks.
  }
}
}
\source{
Redistributed from version 1.0.1 of the GPL-2-licensed R package
\pkg{scalreg}.
}
\references{
This database was used in the R package \pkg{plus}.
}
\examples{
data("sp500")
names(sp500)
dim(sp500$sp500.percent)

attach(sp500)
head(sp500.percent[, 1:5])
detach(sp500)
}
\keyword{datasets}
